Attribute VB_Name = "General"
' Module: General
'  General-purpose procedures.
'
'  Requires Win32.tlb.

Option Explicit

' -----------
'  Constants
' -----------

' Hourglass constants:
Public Enum HourglassConstants
    hgOn = -1  ' or True
    hgOff = 0  ' or False
    hgSave = 1
    hgRestore = 2
    hgRestoreAndOff = 3
End Enum

' ------------
'  Procedures
' ------------

' Appends a shortcut key to the caption of a menu item.
Public Sub AddMenuShortcut(Menu As Menu, Shortcut As String)
    Menu.Caption = Menu.Caption & vbTab & Shortcut
End Sub

' Moves a form to the center of the working area.
Public Sub CenterForm(Form As Form)
    ' Get the coordinates of the screen, excluding the taskbar:
    Dim WorkArea As RECT
    SystemParametersInfo SPI_GETWORKAREA, 0, WorkArea, 0
    
    ' Calculate the new position of the form:
    Dim Left As Long, Top As Long
    Left = (WorkArea.Left + WorkArea.Right - _
        Form.Width \ Screen.TwipsPerPixelX) \ 2
    Top = (WorkArea.Top + WorkArea.Bottom - _
        Form.Height \ Screen.TwipsPerPixelY) \ 2
    
    ' Ensure that the upper-left corner of the form is visible:
    If Left < WorkArea.Left Then Left = WorkArea.Left
    If Top < WorkArea.Top Then Top = WorkArea.Top
    
    ' Move the form:
    SetWindowPos Form.hWnd, 0, Left, Top, 0, 0, _
        SWP_NOSIZE Or SWP_NOZORDER Or SWP_NOACTIVATE
End Sub

' Shows or hides the hourglass mouse pointer.
Public Sub Hourglass(ByVal Display As HourglassConstants)
    Static States(0 To 9) As Long, CurState As Long
    
    Select Case Display
        Case hgOn       ' Show the hourglass
            States(CurState) = States(CurState) + 1
        Case hgOff      ' Hide the hourglass
            States(CurState) = States(CurState) - 1
        Case hgSave     ' Save the state and hide the hourglass
            CurState = CurState + 1
            States(CurState) = 0
        Case hgRestore  ' Restore the previous state
            CurState = CurState - 1
        Case hgRestoreAndOff
            CurState = CurState - 1
            States(CurState) = States(CurState) - 1
        Case Else
            Debug.Assert False
    End Select
    
    If States(CurState) > 0 Then
        Screen.MousePointer = vbHourglass
    Else
        Screen.MousePointer = vbDefault
    End If
End Sub

' Forces a window to refresh itself.
' Call RedrawWindow after modifying window styles.
Public Sub RedrawWindow(ByVal hWnd As Long)
    Const Flags As Long = SWP_NOSIZE Or SWP_NOMOVE Or _
        SWP_NOZORDER Or SWP_NOACTIVATE Or SWP_FRAMECHANGED
    SetWindowPos hWnd, 0, 0, 0, 0, 0, Flags
End Sub

' Highlights all the text within a text box.
Public Sub SelectText(TextBox As TextBox)
    SendMessage TextBox.hWnd, EM_SETSEL, 0, -1
End Sub

' Highlights all the text within a text box
' only if the user just pressed the Tab key.
Public Sub SelectTextUI(TextBox As TextBox)
    If GetKeyState(vbKeyTab) < 0 Then
        SendMessage TextBox.hWnd, EM_SETSEL, 0, -1
    End If
End Sub

' Removes the separator bar from the system menu of a form.
' Call StandardizeSystemMenu to replace
'   +---------------+          +---------------+
'   | Move          |          | Move          |
'   +---------------+   with   | Close  Alt+F4 |
'   | Close  Alt+F4 |          +---------------+
'   +---------------+
Public Sub StandardizeSystemMenu(Form As Form)
    DeleteMenu GetSystemMenu(Form.hWnd, APIFALSE), 5, MF_BYPOSITION
End Sub

' Truncates a string at the first occurrence of the null character.
Public Function ZTrim(Text As String) As String
    Dim Pos As Long
    Pos = InStr(Text, vbNullChar)
    
    If Pos > 0 Then
        ZTrim = Left$(Text, Pos - 1)
    Else
        ZTrim = Text
    End If
End Function
